@@ -63,6 +63,8 @@ def _utc_now():
 class Spider(object):
     """Base class for all spiders"""
 
+    _replace_spiders_postfix_reg_ex = re.compile(r'[-_]spiders$', re.IGNORECASE)
+
     @classmethod
     def _get_crawl_method_arg_names_for_use_as_factors(cls):
         """Returns the list of argument names for
@@ -117,6 +119,25 @@ class Spider(object):
         except Exception as ex:
             raise SpiderMetadataError(cls, ex=ex)
 
+        if not metadata.get('categories', []):
+            #
+            # A spider's fully qualified name will be something like gaming_spiders.miniclip.Spider
+            # so in this case the class name is Spider, module name is miniclip and the package
+            # name is gaming_spiders.
+            #
+            # Spiders are grouped into categories.
+            # A spider can appear in more than one category.
+            # A spider's categories are declared as part of the spider's metadata.
+            # If no categories are declared in a spider's metadata then the
+            # spider's default category is used.
+            #
+            metadata['categories'] = [cls.get_default_category()]
+
+        metadata['absoluteFilename'] = sys.modules[cls.__module__].__file__
+        metadata['fullyQualifiedClassName'] = '{module}.{cls}'.format(
+            module=cls.__module__,
+            cls=cls.__name__)
+
         crawl_method_arg_names = cls._get_crawl_method_arg_names_for_use_as_factors()
         if crawl_method_arg_names is None:
             message_detail = "crawl() method arg names not found"
@@ -194,6 +215,26 @@ class Spider(object):
 
         return metadata
 
+    @classmethod
+    def get_default_category(cls):
+        """A spider's fully qualified name will be something like gaming_spiders.miniclip.Spider
+        A spider's default category is everything up to but not including the first period a
+        spider's fully qualified name.
+
+        get_default_category() simplifies specificiation of metadata when spiders are in
+        mulitple categories. See sample code below.
+
+            def get_metadata(cls):
+               return {
+                   'url': 'https://www.xe.com/?cn=cad',
+                   'categories': [
+                      cls.get_default_category(),
+                      'fx_rates',
+                   ],
+               }
+        """
+        return cls._replace_spiders_postfix_reg_ex.sub('', cls.__module__.split('.')[0])
+
     @classmethod
     def get_metadata(cls):
         """Spider classes should override this method to return
@@ -965,8 +1006,12 @@ class SpiderDiscovery(object):
         r'^\s*(?P<egg_name>.+spiders)-\d+\.\d+\.\d+\-py\d+\.\d+\s*$',
         re.IGNORECASE)
 
-    def discover(self):
+    def __init__(self, samples=False):
+        object.__init__(self)
 
+        self._samples = samples
+
+    def discover(self):
         #
         # find and import all packages that might contain spiders
         #
@@ -978,12 +1023,39 @@ class SpiderDiscovery(object):
                 _logger.info("matched distro for spiders '%s'", egg_name)
                 type(self).load_and_discover_all_spiders_in_package(egg_name)
 
+        if self._samples:
+            importlib.import_module('cloudfeaster.samples')
+            samples_dir = os.path.dirname(sys.modules['cloudfeaster.samples'].__file__)
+            def filter(filename): return(filename.endswith('.py') and not filename.startswith('__'))
+            sample_spiders = [filename[:-len('.py')] for filename in os.listdir(samples_dir) if filter(filename)]
+            for sample_spider in sample_spiders:
+                importlib.import_module('cloudfeaster.samples.{spider}'.format(spider=sample_spider))
+
         #
         # with all packages loaded that might contain spiders, find all
         # the concrete subclasses of ```cloudfeaster.spider.Spider```
         # which will be the spiders we're interested in
         #
-        return self._find_concrete_spider_classes(Spider)
+        concrete_spider_classes = self._find_concrete_spider_classes(Spider)
+
+        #
+        # now for some fancy formatting of the results
+        #
+        rv = {}
+        for concrete_spider_class in concrete_spider_classes:
+            metadata = concrete_spider_class.get_validated_metadata()
+            fully_qualified_class = metadata['fullyQualifiedClassName']
+            spider = fully_qualified_class.split('.')[-2]
+
+            for category in metadata['categories']:
+                if category not in rv:
+                    rv[category] = {}
+                rv[category][spider] = metadata
+
+        #
+        # all done!
+        #
+        return rv
 
     def _find_concrete_spider_classes(self, base_class):
         base_msg = "looking for concrete spider classes of base class '%s.%s'" % (
@@ -992,22 +1064,24 @@ class SpiderDiscovery(object):
         )
         _logger.info(base_msg)
 
-        rv = {}
+        concrete_spider_classes = []
         for sub_class in base_class.__subclasses__():
-            full_sub_class_name = '%s.%s' % (sub_class.__module__, sub_class.__name__)
+            fully_qualified_class_name = '%s.%s' % (sub_class.__module__, sub_class.__name__)
 
-            _logger.info("%s - assessing '%s'", base_msg, full_sub_class_name)
+            _logger.info("%s - assessing '%s'", base_msg, fully_qualified_class_name)
 
             if not sub_class.__subclasses__():
-                _logger.info("%s - identified concrete class '%s'", base_msg, full_sub_class_name)
+                msg_fmt = "{base_msg} - identified concrete class '{class_name}'"
+                msg = msg_fmt.format(base_msg=base_msg, class_name=fully_qualified_class_name)
+                _logger.info(msg)
 
-                rv[full_sub_class_name] = sub_class.get_validated_metadata()
+                concrete_spider_classes.append(sub_class)
             else:
-                _logger.info("%s - identified abstract class '%s'", base_msg, full_sub_class_name)
+                _logger.info("%s - identified abstract class '%s'", base_msg, fully_qualified_class_name)
 
-                rv.update(self._find_concrete_spider_classes(sub_class))
+                concrete_spider_classes.extend(self._find_concrete_spider_classes(sub_class))
 
-        return rv
+        return concrete_spider_classes
 
     @classmethod
     def load_and_discover_all_spiders_in_package(cls, spider_package_name):
